#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QGLWidget>
#include <QMutex>

#include <Box2D/Box2D.h>

#include "world.h" // testing only


class GLWidget : public QGLWidget
{
    Q_OBJECT

public:
    GLWidget(QWidget *parent = 0);
    ~GLWidget();


    void setWorld( World *world, QMutex *worldMutex );
    
    
    /**
     * Converts pixels on the screen into meters in Box2D
     */
    b2Vec2 screenToWorld( const QPointF& screenCoordinate );
    
    void drawCreature( Creature *creature );
    
    void drawB2Body( const b2Body *body );
    
    void pan( const QPointF& amount );
    void zoom( double factor );
    
    void animate( bool enable = true );
    void enableWorldRender( bool enable = true );
    bool isAnimated();
    
public slots:

protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *event);
    virtual void wheelEvent(QWheelEvent *event);

private:
    QPoint m_lastPos;
    QMutex *m_worldMutex;
    World *m_world;
    
    float m_zoomLevel;
    b2Vec2 m_pan;
    
    bool m_animationEnabled;
    bool m_worldRenderingEnabled;
    
    QTimer *m_updateTimer;
};

#endif
