#include "plant.h"
#include "world.h"
#include <QVector>


Plant::Plant(b2World* box2DWorld, const CreatureConfiguration& configuration, World* parent): Creature(box2DWorld, configuration, parent)
{
    b2BodyDef bodydef;
    bodydef.position.Set(configuration.position.x, configuration.position.y);
    bodydef.type=b2_dynamicBody;
    
    m_body = m_box2DWorld->CreateBody(&bodydef);
    m_body->SetUserData(this);
    
    m_energy = configuration.size * 10;
    m_initialEnergy = m_energy;
    
    m_createShape(configuration.size);
}


Plant::Plant( const b2Vec2& pos, float size, b2World *box2DWorld, World *parent ) : Creature( pos, size, box2DWorld, parent )
{
    b2BodyDef bodydef;
    bodydef.position.Set(pos.x, pos.y);
    bodydef.type=b2_dynamicBody;
    
    m_body = m_box2DWorld->CreateBody(&bodydef);
    m_body->SetUserData(this);
    
    m_growthRate = 1.000001;
    
    m_energy = size * 10;
    
    m_createShape(size);
    
    // plants don't need sensors
//     createSensor( 2 * size );
}

Creature::Type Plant::type()
{
    return Creature::PLANT;
}

void Plant::Step()
{
    Creature::Step();
    
    m_energy = m_body->GetMass() / 50;
    
    // reproduction
    if( canReproduce() )
    {
	if( qrand() > RAND_MAX * m_mutationRate ) // not every seed succeeds
	    reproduce( this );
	else m_reproductionTime = 0;
    }
}

float Plant::takeEnergy(Creature* taker)
{
    float e = Creature::takeEnergy(taker);
    e += taker->mass()/10;
    shrink(0.98);
    return e;
}

void Plant::m_createShape(double size)
{
    const float DEG2RAD = 3.14159/180;
    QVector<b2Vec2> vertices;
    for (int i=0; i < 360; i += 45)
    {
        float degInRad = i*DEG2RAD;
        vertices.append( b2Vec2(cos(degInRad)*size,sin(degInRad)*size) );
    }
    b2PolygonShape shape;
    shape.Set(vertices.data(), vertices.length());

    b2FixtureDef fixturedef;
    fixturedef.shape = &shape;
    fixturedef.density = 100.0;
    fixturedef.restitution = 0;
    m_body->CreateFixture(&fixturedef);
}

