#include "predator.h"
#include "world.h"
#include <QVector>


Predator::Predator(b2World* box2DWorld, const CreatureConfiguration& configuration, World* parent): Creature(box2DWorld, configuration, parent)
{
    b2BodyDef bodydef;
    bodydef.position.Set(configuration.position.x, configuration.position.y);
    bodydef.type=b2_dynamicBody;
    
    m_body = m_box2DWorld->CreateBody(&bodydef);
    m_body->SetUserData(this);
    
    m_createShape(configuration.size);
}

Predator::Predator( const b2Vec2& pos, float size, b2World *box2DWorld, World *parent ) : Creature( pos, size, box2DWorld, parent )
{
    b2BodyDef bodydef;
    bodydef.position.Set(pos.x, pos.y);
    bodydef.type=b2_dynamicBody;
    
    m_body = m_box2DWorld->CreateBody(&bodydef);
    m_body->SetUserData(this);
    
    m_createShape(size);
}

Creature::Type Predator::type()
{
    return Creature::Creature::PREDATOR;
}

void Predator::Step()
{
    Creature::Step();
    
    bool busy = false;
    
    // if energy low, check if Herbivores nearby
    if( energyPercentage() < 0.50 )
    {
	Herbivore *herbivore = (Herbivore *) getClosestVisible(HERBIVORE);
	if( herbivore )
	{
	    busy = true;
	    // go eat
	    b2Vec2 fromTarget = herbivore->position() - m_body->GetPosition();
	    float desiredAngle = atan2( fromTarget.y, fromTarget.x );
	    turnTo( desiredAngle );
	    go(150);
	}
    }
    
    // if all is well, look for predators to reproduce with
    if( canReproduce() )
    {
	Predator *predator = (Predator *) getClosestVisible(PREDATOR);
	if( predator )
	{
	    busy = true;
	    // go eat
	    b2Vec2 fromTarget = predator->position() - m_body->GetPosition();
	    float desiredAngle = atan2( fromTarget.y, fromTarget.x );
	    turnTo( desiredAngle );
	    go(50);
	}
    }
    
    
    // touchy feely
    if( !m_touchingCreatures.isEmpty() )
    {
	foreach( Creature *creature, m_touchingCreatures )
	{
	    switch( creature->type() )
	    {
		case PREDATOR:
		    // check for reproduction ability
		    if( canReproduce() && creature->canReproduce() )
		    {
			busy = true;
			reproduce( creature );
		    }
		    break;
		    
		case HERBIVORE:
		    // eat if lack energy
		    if( energyPercentage() < 1.20 )
		    {
			busy = true;
			creature->paralyze();
			creature->shrink();
			m_energy += creature->takeEnergy(this);
		    }
		    break;
		    
		case PLANT:
		default:
		    break;
	    }
	}
    }
    
    if( !busy )
    {
	explore();
    }
}


void Predator::explore()
{
    Creature::explore();
    
    if( energyPercentage() > 1.2 ) return;
    
    Herbivore *herbivore = (Herbivore *) getClosestVisible(HERBIVORE);
    if( herbivore )
    {
	b2Vec2 toTarget = herbivore->position() - m_body->GetPosition();
	float desiredAngle = atan2( toTarget.y, toTarget.x );
	turnTo( desiredAngle );
	go(50);
    }
}


// b2Vec2 Predator::forceDirection()
// {
//     b2Vec2 front, back;
//     b2PolygonShape* shape = (b2PolygonShape*)body()->GetFixtureList()->GetShape();
//     int vertexCount = shape->GetVertexCount();
//     
//     front = shape->GetVertex(0);
//     back = shape->GetVertex(2) + 0.5 * ( shape->GetVertex(2) - shape->GetVertex(1) );
//     
//     b2Vec2 vec = back - front;
//     vec = 1./sqrt(vec.x*vec.x + vec.y*vec.y) * vec;
//     
//     // rotate vector to fit the body orientation
//     vec.x = vec.x * cos( body()->GetAngle() ) - vec.y * sin( body()->GetAngle() );
//     vec.y = vec.x * sin( body()->GetAngle() ) + vec.y * cos( body()->GetAngle() );
//     
//     return vec;
// }


void Predator::m_createShape(double size)
{
    const float DEG2RAD = 3.14159/180;
    QVector<b2Vec2> vertices;
    for (int i=0; i < 360; i += 120)
    {
        float degInRad = i*DEG2RAD;
        vertices.append( b2Vec2(cos(degInRad)*size,sin(degInRad)*size) );
    }
    b2PolygonShape shape;
    shape.Set(vertices.data(), vertices.length());

    b2FixtureDef fixturedef;
    fixturedef.shape=&shape;
    fixturedef.density=0.5;
    m_body->CreateFixture(&fixturedef);
    m_body->SetAngularDamping( 0.5 );
    m_body->SetLinearDamping( 0.5 );
    
    createSensor( 10 * size );
}

